﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint;
using System.Globalization;
using Microsoft.SharePoint.Utilities;
using System.Web.UI.WebControls;
using System.Diagnostics;
using System.Web;

namespace LanguageConfigurationSolution
{
    public class RootLandingPage : Microsoft.SharePoint.WebControls.UnsecuredLayoutsPageBase
    {

        private const string QualityValuePrefix = ";q=";

        protected override void OnLoad(EventArgs e)
        {
            SPSecurity.RunWithElevatedPrivileges(delegate
            {
                SPSite site = new SPSite(SPContext.Current.Site.ID);
                string targetUrl = this.GetRedirectTargetUrl(site);
                if (!string.IsNullOrEmpty(targetUrl))
                {
                    Redirect(targetUrl);
                }
                else
                {
                    try
                    {
                        SPWeb web = site.RootWeb.Webs[0];
                        Redirect(web.ServerRelativeUrl);
                    }
                    catch
                    {
                        Label errorlabel = new Label();
                        errorlabel.Text = "No redirect for the language could take place";
                        Controls.Add(errorlabel);
                        EventLog.WriteEntry(this.GetType().FullName, "No redirect for the language could take place", EventLogEntryType.Warning);
                    }
                }
            });
        }

        private void Redirect(string targetUrl)
        {
            // Use default Sharepoint behaviour (302) when infopath and/or frontpage, otherwise can't connect.
            string userAgent = HttpContext.Current.Request.UserAgent.ToLowerInvariant();
            if (userAgent.Contains("infopath") || userAgent.Contains("frontpage"))
            {
                SPUtility.Redirect(targetUrl, SPRedirectFlags.Default, Context);
            }
            else
            {

                HttpContext.Current.Response.AddHeader("Location", targetUrl);
                HttpContext.Current.Response.StatusCode = 301;
                HttpContext.Current.Response.Status = "301 Moved Permanently";
            }

        }

        private string GetSubWebUrl(SPWeb web)
        {

            return web.ServerRelativeUrl;
        }

        public const string DEFAULTSUBWEBPROPERTYNAME = "LanguageConfigurationDefaultSubWeb";
        public const string DEFAULTSUBWEBCOOKIENAME = "DefaultLanguageSubWeb";

        private string GetRedirectTargetUrl(SPSite site)
        {
            HttpCookie cookie = Page.Request.Cookies[DEFAULTSUBWEBCOOKIENAME];
            if (cookie != null)
            {
                using (SPWeb web = site.RootWeb.Webs[cookie.Value])
                {
                    return GetSubWebUrl(web);
                }
            }

            foreach (string language in GetUserLanguages())
            {
                try
                {
                    CultureInfo ci = new CultureInfo(language);

                    #region Find the Language SubWeb with the TwoLetterIsoLanguageName
                    try
                    {
                        SPWeb web = site.RootWeb.Webs[ci.TwoLetterISOLanguageName];
                        if (web != null && web.Exists)
                        {
                            return GetSubWebUrl(web);
                        }
                    }
                    catch
                    {
                    }
                    #endregion

                    #region Find the Language SubWeb which has regionalsettings with the samen TwoLetterIsoLanguageName
                    foreach (SPWeb web in site.RootWeb.Webs)
                    {
                        if (web.Locale.TwoLetterISOLanguageName == ci.TwoLetterISOLanguageName)
                        {
                            return web.ServerRelativeUrl;
                        }
                    }
                    #endregion
                }
                catch
                {
                    // Unable to parse the given language-identifier
                }
            }

            #region Determine the default Language SubWeb based upon an CustomProperty of the RootWeb
            try
            {
                string LanguageConfigurationDefaultSubWeb = site.RootWeb.Properties[DEFAULTSUBWEBPROPERTYNAME];
                if (!string.IsNullOrEmpty(LanguageConfigurationDefaultSubWeb))
                {
                    using (SPWeb web = site.RootWeb.Webs[LanguageConfigurationDefaultSubWeb])
                    {
                        return GetSubWebUrl(web);
                    }
                }
            }
            catch
            { }
            #endregion

            #region Determine the default Language SubWeb based upon the Regional Settings of the RootWeb
            try
            {
                foreach (SPWeb web in site.RootWeb.Webs)
                {
                    if (web.Locale.TwoLetterISOLanguageName == site.RootWeb.Locale.TwoLetterISOLanguageName)
                    {
                        return GetSubWebUrl(web);
                    }
                }
            }
            catch
            { }
            #endregion

            return null;
        }


        /// <summary>
        /// Retrieve the Preferred UserLanguages from the webbrowser client
        /// </summary>
        /// <returns>An array of UserLanguages (</returns>
        private string[] GetUserLanguages()
        {
            string[] browserPrefLanguages = Page.Request.UserLanguages;
            if (null != browserPrefLanguages)
            {
                // Strips off the quality parameters. Thus quality remains unused/ignored.
                int qualityIndexPos = -1;
                for (int i = 0; i < browserPrefLanguages.Length; i++)
                {
                    qualityIndexPos = browserPrefLanguages[i].IndexOf(QualityValuePrefix, StringComparison.Ordinal);
                    if (qualityIndexPos > 0)
                    {
                        browserPrefLanguages[i] = browserPrefLanguages[i].Substring(0, qualityIndexPos);
                    }
                }
            }
            return browserPrefLanguages;
        }

    }
}
